﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace PojeClient {
    // AIを作る際にあると便利な関数　
    class AITools {

        // 盤 board で手 actionを打った場合に，どういう攻撃になるか．      
        // updateSrcBoard = true の場合，引数のsrcBoard自体をいじる．falseの場合はコピーしてからいじる（安全）
        // 新しく追加される配珠（普通は，予測にすぎない）を加えられるようにした
        public static AITool_Result calcTotalGainWithNexts(Board board, Action action, bool updateSrcBoard, int col1, int col2) {
            if (updateSrcBoard == false) board = new Board(board);
            List<Point> drops = board.transit1_ActionDrop(action);
            int colorVanishCount = board.transit2_calcVanishPoints(drops);
            if (colorVanishCount == 0) {
                // その後のboardを使うことがあるので，配珠をずらす（次が読めるようにする）．
                board.pushNextColors(new int[2] { col1, col2 });
                return new AITool_Result(false, 0, 0, 0, action, board);
            }

            // 消えなくなるまでループする
            int chainCount = 0;
            int totalGain = 0;
            int jamVanishCount = 0;
            do {
                chainCount++;
                jamVanishCount += board.getVanishPoints().Count - colorVanishCount; // 消える箇所の全体数 - 消える色珠数 = 消えるJamの数
                totalGain += JamInfo.getGainJamCount(chainCount, colorVanishCount);
                board.transit3_Vanish();
                drops = board.transit1_ChainDrop();  //連鎖中は行動なし，邪魔珠落下もなし
                colorVanishCount = board.transit2_calcVanishPoints(drops);
            } while (colorVanishCount > 0);

            // その後のboardを使うことがあるので，配珠をずらす（次が読めるようにする）．
            board.pushNextColors(new int[2] { col1, col2 });
            return new AITool_Result(true, totalGain, chainCount, jamVanishCount, action, board);
        }

        // 盤 board で手 actionを打った場合に，どういう攻撃になるか．      
        // updateSrcBoard = true の場合，引数のsrcBoard自体をいじる．falseの場合はコピーしてからいじる（安全）
        public static AITool_Result calcTotalGain(Board board, Action action, bool updateSrcBoard) {
            return calcTotalGainWithNexts(board, action, updateSrcBoard, 0, 0);
        }




        // あるターン数の中で最大の攻撃ができる手（の一手目）を選ぶ．
        // この結果は，その盤（とnexts）が持っている，攻撃ポテンシャルの一つ．
        public static AITool_Result calcBestAttackAction(Board board, List<Action> actions, int restTurn) {
            AITool_Result bestResult = null;

            if (restTurn == 1) {
                // このターンで発動しなければいけない場合
                // actionがない場合（すでに死んでいる）はnullが返る
                foreach (Action a in actions) {
                    AITool_Result result = calcTotalGain(board, a, false);
                    if (bestResult == null) {
                        bestResult = result;
                    } else if (result.Gain > bestResult.Gain) {
                        bestResult = result;
                    }
                }
            } else {
                // まだターンに余裕がある場合は，発動しない手についてもその先を読む（発動するならするでよい）
                foreach (Action a in actions) {
                    AITool_Result result = calcTotalGain(board, a, false);

                    // 発動する手なら，result がその最終価値．　そうでないなら，次の局面での calcBestAttackActionが最終価値．
                    if (result.IsVanish == false) {
                        // その時点で打つ手がない場合，最悪評価
                        if (result.NextBoard.createPossibleActions().Count == 0) {
                        } else {
                            // 手があるならそのポテンシャルを評価
                            result = calcBestAttackAction(result.NextBoard, result.NextBoard.getPossibleActions(), restTurn - 1);
                            // 選んだ手は a なのでそれを格納
                            result.Action = a;
                        }
                    }

                    // そのうえで，最善のものを比較格納
                    if (bestResult == null) {
                        bestResult = result;
                    } else if (result.Gain > bestResult.Gain) {
                        bestResult = result;
                    }
                }
            }

            return bestResult;



        }





        // 邪魔珠がいつ何個降ってくるか，現在相手が連鎖中の場合も含めて確定させる
        public static JamInfo calcOpponentsAttack(Board srcBoard, JamInfo srcJam, int turn) {
            int chainCount = srcBoard.getChainCount();
            // 相手が連鎖中でない場合は，そのままでよい
            if (chainCount == 0) return srcJam;

            JamInfo jam = new JamInfo(srcJam);
            Board board = new Board(srcBoard);

            // 消えなくなるまでループする
            while (true) {
                List<Point> drops = board.transit1_ChainDrop();  //連鎖中は行動なし，邪魔珠落下もなし
                int colorVanishCount = board.transit2_calcVanishPoints(drops);
                if (colorVanishCount == 0) {
                    // ターンの確定
                    jam.setJamTurn(turn+2);
                    break;
                }
                chainCount++;
                jam.addJam(JamInfo.getGainJamCount(chainCount, colorVanishCount));
                board.transit3_Vanish();
                turn++;
            }
            return jam;
        }


        // その手で落としたときに，周囲に同じ色があるか．弱いAIが連鎖しやすくするためのもの
        public static int calcConnectCount(Board srcBoard, Action action) {
            int count = 0;
            Point[] p = action.getDropPoints();
            for (int i = 0; i < 2; i++) {
                if (p[i].Y <= Settings.BoardHeight) {
                    for (int r = 0; r < 4; r++) {
                        if (srcBoard.getStone(p[i].X + Settings.RotateX[r], p[i].Y + Settings.RotateY[r]) == action.getColor(i)) count++;
                    }
                }
            }
            return count;
        }

        // あるボードについての形の良さ
        // action付きのものにくらべて少し重いけど，読んだ後の盤面にもつかえる
        public static int evalBoard(Board srcBoard) {
            int ev = 0;
            for (int y = 1; y <= Settings.BoardHeight; y++) {
                for (int x = 1; x <= Settings.BoardWidth; x++) {
                    int c = srcBoard.getStone(x, y);
                    if (c > 0) {
                        // 周囲が同じ色なら5点（２回カウントされることに注意）
                        for (int r = 0; r < 4; r++) {
                            if (srcBoard.getStone(x + Settings.RotateX[r], y + Settings.RotateY[r]) == c) ev += 5;
                        }
                        // ただし上下，左右が同じ（３連）なら -4点で少しキャンセル
                        if (srcBoard.getStone(x - 1, y) == c && srcBoard.getStone(x + 1, y) == c) ev -= 4;
                        if (srcBoard.getStone(x, y+1) == c && srcBoard.getStone(x, y-1) == c) ev -= 4;

                        if (y < 8) {
                            // ななめ上が同じ色なら12点 （カギ型は除く）
                            if (srcBoard.getStone(x - 1, y + 1) == c && srcBoard.getStone(x - 1, y) != c && srcBoard.getStone(x, y + 1) != c) ev += 12;
                            if (srcBoard.getStone(x + 1, y + 1) == c && srcBoard.getStone(x + 1, y) != c && srcBoard.getStone(x, y + 1) != c) ev += 12;

                            // 同じ色がたてに別の色を挟んだ場合（ただし挟んだ石の片方には石が，片方には同色か空地があること）
                            if (srcBoard.getStone(x, y + 2) == c) {
                                if ((srcBoard.getStone(x + 1, y + 1) > 0 && (srcBoard.getStone(x - 1, y + 1) == c || srcBoard.getStone(x - 1, y + 1) == 0)) ||
                                     (srcBoard.getStone(x - 1, y + 1) > 0 && (srcBoard.getStone(x + 1, y + 1) == c || srcBoard.getStone(x + 1, y + 1) == 0))) ev += 10;
                            }
                        }
                    }
                }
            }
            //高さに差があるのはあまりよくない
            for (int x = 1; x < Settings.BoardWidth; x++) {
                ev -= ((srcBoard.getHeight(x) - srcBoard.getHeight(x + 1)) * (srcBoard.getHeight(x) - srcBoard.getHeight(x + 1)));
            }

            //そもそも低いほうがよい
            for (int x = 1; x < Settings.BoardWidth; x++) {
                ev -= ((srcBoard.getHeight(x) + 4) * (srcBoard.getHeight(x) + 4)) / 10;
            }

            return ev;
        }


        // 盤上，残り何マス空いてるか．緊急度の目安．
        public static int getRestCount(Board board) {
            int rest = 0;
            for (int x = 1; x <= Settings.BoardWidth; x++) {
                rest += Settings.BoardHeight - board.getHeight(x);
            }
            return rest;
        }










    }


    // いろんなものを返したい場合の構造体
    // Ver 802 Chain　を追加
    class AITool_Result {
        public bool IsVanish;
        public int Gain;
        public int Chain;
        public int JamVanishCount;
        public Action Action;
        public Board NextBoard;

        public AITool_Result(bool isVanish, int gain, int chain, int jamVanishCount, Action a, Board b) {
            IsVanish = isVanish;
            Gain = gain;
            Chain = chain;
            JamVanishCount = jamVanishCount;
            Action = a;
            NextBoard = b;
        }
    }



}
