﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace PojeClient {
    // 3手+同色珠　以内で打てる量を最大にするような，発動しない初手を選ぶ．一定の条件下で実際に攻撃する．
    // ちょっと重いのでスレッド並列化した
    class AI_ChargeAndFire4 : Player {
        private int fMyIndex;
        private int fTimeLimit;
        private Action[] fAnswerBox;
        private Random fRand;


        public String getName() {
            return "AI_CF4";
        }

        public void newGame(int yourIndex) {
            fRand = new Random(yourIndex + (new Random()).Next());
            fMyIndex = yourIndex;
            fSummary = new String[100];
        }

        public void endGame(int result) {
        }

        public void setParam(int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
        }

        public void makeAction(object currentState) {
            State state = (State)currentState;
            Board myBoard = state.getBoard(fMyIndex);
            Board oppBoard = state.getBoard(1 - fMyIndex);
            fPossibleActions = myBoard.createPossibleActions();

            int restTurn = int.MaxValue;
            // Case 1. 埋まりそうな場合．
            if (AITools.getRestCount(myBoard) < 14) restTurn = 3;
            // Case 2. 相手の攻撃がある程度あって近い場合．
            fHisAttack = AITools.calcOpponentsAttack(oppBoard, myBoard.getJamInfo(), state.getTurn());   // 今自分の上にある　＋　相手が連鎖中のもの　を含めた邪魔珠．
            fHisAttack.deleteJam(oppBoard.getJamInfo().getTotalCount());                          // そこから，今相手の上にあるものを引く．これで実質攻撃される数と時間が分かる．

            if (fHisAttack.getCountTill(state.getTurn() + 3) > 5) restTurn = 3;
            if (fHisAttack.getCountTill(state.getTurn() + 2) > 5) restTurn = 2;
            if (fHisAttack.getCountTill(state.getTurn() + 1) > 5) restTurn = 1;

            //// 危機が３手以内に迫っているなら，打てる中で最も攻撃力の高い手を打つ．(Fire)
            if (restTurn <= 3) {
                AITool_Result result2 = AITools.calcBestAttackAction(myBoard, fPossibleActions, restTurn);
                if (result2.Gain + result2.JamVanishCount > 0) {
                    fAnswerBox[0] = result2.Action;
                    Logger.ailog("CaF4 : reaction = "+result2.Gain+" enemy's attack="+fHisAttack.getTotalCount());
                } else {
                    // 消せないなら，ランダムに置く
                    fAnswerBox[0] = fPossibleActions[fRand.Next(fPossibleActions.Count)];
                }
                return;
            }

            // そうでないなら，将来の攻撃力をベースに，それが同じなら適当にくっつくように，消さない手を打つ． (Charge)
            // 各手に対してscoreを計算するスレッドを起動
            fTempBoard = myBoard;
            Thread[] threads = new Thread[fPossibleActions.Count];
            fActionScore = new int[fPossibleActions.Count];
            for (int i = 0; i < fPossibleActions.Count; i++) {
                threads[i] = new Thread(new ParameterizedThreadStart(calcScore));
                threads[i].Start(i);
            }

            // スレッドが終了するまで待ち，最良のものを選ぶ
            int bestScore = int.MinValue;
            int bestIndex = -1;
            for (int i = 0; i < fPossibleActions.Count; i++) {
                threads[i].Join();
                if (bestScore < fActionScore[i]) {
                    bestScore = fActionScore[i];
                    bestIndex = i;
                }
            }
            fAnswerBox[0] = fPossibleActions[bestIndex];

            Logger.ailog("CaF4 : " + fSummary[bestIndex]);
        }



        private Board fTempBoard;
        private int[] fActionScore;
        private String[] fSummary;
        private JamInfo fHisAttack;
        private List<Action> fPossibleActions;

        private void calcScore(object actionIndex) {
            Action a = fPossibleActions[(int)actionIndex];
            int score;

            
            AITool_Result result = AITools.calcTotalGain(fTempBoard, a, false);         // その手の攻撃結果
            Board nextBoard = result.NextBoard;

            if (result.IsVanish == true) {
                // 消してしまう場合はそれでも小さいものを選ぶ
                score = -result.Gain;
                if (result.Gain > 450) {
                    score = 1000 * result.Gain;   // （このクラスとしては）十分大きい攻撃ならしてしまう
                    fSummary[(int)actionIndex] = "Enough = "+result.Gain;
                }
                if (result.Gain < fHisAttack.getTotalCount()) score = -result.Gain;     // ただそれでも負けるようならやっぱり攻撃しないで粘る
            } else {
                // ポテンシャル計算
                int maxPot = 0;
                for (int color = 1; color <= Settings.NumColor; color++) {
                    // 消去なし落下の場合の，最後のツモを１色珠にする
                    nextBoard.setNextColor(color, Settings.NextOpenTurns * 2-2);

                    // その場合のポテンシャルを得る．Nextはあと2+1個分見えている
                    AITool_Result result3 = AITools.calcBestAttackAction(nextBoard, nextBoard.createPossibleActions(), Settings.NextOpenTurns - 0);

                    if (result3.Gain > maxPot) maxPot =  result3.Gain;
                }

                // ポテンシャルを主に，適当にくっつくようなスコア計算
                score = fRand.Next(100) + 70 * AITools.evalBoard(nextBoard) + 200 * maxPot;
                fSummary[(int)actionIndex] = "potential = " + maxPot;
            }

            fActionScore[(int)actionIndex] = score;
        }




    }



}
