﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace PojeClient {
    // ベースは CaF4 だが，常に最大攻撃をするとは限らず，本体を壊さない程度で弱打をすることがある
    // 再帰部分が面倒なので，OpenCount=3 限定
    class AI_SmartReaction : Player {
        private int fMyIndex;
        private int fTimeLimit;
        private Action[] fAnswerBox;
        private Random fRand;

        public String getName() {
            return "AI_SmartReaction";
        }

        public void newGame(int yourIndex) {
            fRand = new Random(yourIndex + (new Random()).Next());
            fMyIndex = yourIndex;
            fSummary = new String[100];
        }

        public void endGame(int result) {
        }

        public void setParam(int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
        }

        public void makeAction(object currentState) {
            State state = (State)currentState;
            Board myBoard = state.getBoard(fMyIndex);
            Board oppBoard = state.getBoard(1 - fMyIndex);
            fPossibleActions = myBoard.createPossibleActions();

            // そろそろ埋まりそうな場合は打てる中での最大手を打つ．
            if (AITools.getRestCount(myBoard) < 18) {
                AITool_Result result2 = AITools.calcBestAttackAction(myBoard, fPossibleActions, Settings.NextOpenTurns);
                fAnswerBox[0] = result2.Action;
                Logger.ailog("SR : Final Attack = " + result2.Gain);
                return;
            }

            // 相手の攻撃の計算
            fHisAttack = AITools.calcOpponentsAttack(oppBoard, myBoard.getJamInfo(), state.getTurn());
            fHisAttack.deleteJam(oppBoard.getJamInfo().getTotalCount());

            fHisAttackTurn1 = fHisAttack.getCountTill(state.getTurn() + 1);
            fHisAttackTurn2 = fHisAttack.getCountTill(state.getTurn() + 2);
            fHisAttackTurn3 = fHisAttack.getCountTill(state.getTurn() + 3);

            int restTurn = int.MaxValue;
            // 大きめの攻撃が来ているときは，本線で返す
            if (fHisAttackTurn3 > 17) restTurn = 3;
            if (fHisAttackTurn2 > 17) restTurn = 2;
            if (fHisAttackTurn1 > 17) restTurn = 1;

            if (restTurn <= 3) {
                AITool_Result result2 = AITools.calcBestAttackAction(myBoard, fPossibleActions, restTurn);
                if (result2.Gain + result2.JamVanishCount > 0) {
                    fAnswerBox[0] = result2.Action;
                    Logger.ailog("SR : Reaction to Big Attack = " + result2.Gain + " (enemy's =" + fHisAttack.getTotalCount() + ")");
                } else {
                    // 消せないなら，ランダムに置く
                    fAnswerBox[0] = fPossibleActions[fRand.Next(fPossibleActions.Count)];
                }
                return;
            }


            // 各手に対してscoreを計算するスレッドを起動
            fTempBoard = myBoard;
            Thread[] threads = new Thread[fPossibleActions.Count];
            fActionScore = new int[fPossibleActions.Count];
            for (int i = 0; i < fPossibleActions.Count; i++) {
                threads[i] = new Thread(new ParameterizedThreadStart(calcScore));
                threads[i].Start(i);
            }

            // スレッドが終了するまで待ち，score最良のものを選ぶ
            int bestScore = int.MinValue;
            int bestIndex = -1;
            for (int i = 0; i < fPossibleActions.Count; i++) {
                threads[i].Join();
                if (bestScore < fActionScore[i]) {
                    bestScore = fActionScore[i];
                    bestIndex = i;
                }
            }
            fAnswerBox[0] = fPossibleActions[bestIndex];
            Logger.ailog("SR : " + fSummary[bestIndex]);
        }



        private Board fTempBoard;
        private int[] fActionScore;
        private String[] fSummary;
        private JamInfo fHisAttack;
        private int fHisAttackTurn1, fHisAttackTurn2, fHisAttackTurn3;

        private List<Action> fPossibleActions;

        // 各初手に対して，その先を読む
        private void calcScore(object actionIndex) {
            int bestScore = int.MinValue;

            // 初手の結果
            Action a1 = fPossibleActions[(int)actionIndex];
            AITool_Result result1st = AITools.calcTotalGain(fTempBoard, a1, false);
            Board board2nd = result1st.NextBoard;
            List<Action> actions2nd = board2nd.createPossibleActions();

            // 最後に足す connectScore
            int connectScore = AITools.evalBoard(board2nd);

            foreach (Action a2 in actions2nd) {
                // 2手目の結果
                AITool_Result result2nd = AITools.calcTotalGain(board2nd, a2, false);
                // ２回目の消去ならそこで終わり，スコア計算
                if (result1st.IsVanish == true && result2nd.IsVanish == true) {
                    bestScore = updateScoreDouble((int)actionIndex, bestScore, result1st.IsVanish, result1st, 1, result2nd, 2);
                    continue;
                }
                Board board3rd = result2nd.NextBoard;
                List<Action> actions3rd = board3rd.createPossibleActions();

                foreach (Action a3 in actions3rd) {
                    // 3手目の結果
                    AITool_Result result3rd = AITools.calcTotalGain(board3rd, a3, false);
                    // ２回目の消去ならそこで探索打ち切り，スコア計算
                    if (result1st.IsVanish == true && result3rd.IsVanish == true) {
                        bestScore = updateScoreDouble((int)actionIndex, bestScore, result1st.IsVanish, result1st, 1, result3rd, 3);
                        continue;
                    }
                    if (result2nd.IsVanish == true && result3rd.IsVanish == true) {
                        bestScore = updateScoreDouble((int)actionIndex, bestScore, result1st.IsVanish, result2nd, 2, result3rd, 3);
                        continue;
                    }
                    Board board4th = result3rd.NextBoard;

                    // 1-3手で消したことがあれば記憶する（ここまで来てるならあっても１回）
                    AITool_Result priorVanish = null;
                    int priorVanishTurn = -1;
                    if (result1st.IsVanish == true) {
                        priorVanish = result1st;
                        priorVanishTurn = 1;
                    } else if (result2nd.IsVanish == true) {
                        priorVanish = result2nd;
                        priorVanishTurn = 2;
                    } else if (result3rd.IsVanish == true) {
                        priorVanish = result3rd;
                        priorVanishTurn = 3;
                    }

                    bool noVanish4th = true;
                    for (int color = 1; color <= Settings.NumColor; color++) {
                        board4th.setNextColor(color, 0);
                        List<Action> actions4th = board4th.createPossibleActions();
                        foreach (Action a4 in actions4th) {
                            //4手目の結果
                            AITool_Result result4th = AITools.calcTotalGain(board4th, a4, false);
                            ///スコア計算
                            if (result4th.IsVanish == true) {
                                noVanish4th = false;
                                if (priorVanish != null) {
                                    // ２回目の消去のとき
                                    bestScore = updateScoreDouble((int)actionIndex, bestScore, result1st.IsVanish, priorVanish, priorVanishTurn, result4th, 4);
                                } else {
                                    // 初めての消去のとき
                                    bestScore = updateScoreSingle((int)actionIndex, bestScore, result1st.IsVanish, result4th, 4);
                                }
                            }
                        }
                    }

                    // 最後には消えないけどそこまでで１回消えてるとき
                    if (priorVanish != null && noVanish4th == true) {
                        bestScore = updateScoreSingle((int)actionIndex, bestScore, result1st.IsVanish, priorVanish, priorVanishTurn);
                    }
                    // 一回も消えないときは，３手目終了時点での珠のつながり具合を評価する
                    if (priorVanish == null && noVanish4th == true) {
                        int connected = AITools.evalBoard(board4th);
                        int score = MARGIN_NOVANISH + fRand.Next(100) + 10 * connected;
                        if (score > bestScore) {
                            bestScore = score;
                            fSummary[(int)actionIndex] = "No vanish connectScore = " + connected;
                        }
                    }

                }
            }
            fActionScore[(int)actionIndex] = bestScore + connectScore * 30;
        }

        const int MARGIN_ENOUGHATTACK = 1000000;
        const int MARGIN_RESTTURN1 = 100000;
        const int MARGIN_NORMALPOTENTIAL = 10000;
        const int MARGIN_NORMALPOTENTIAL_CLEARNING = 10000-1500;
        const int MARGIN_NOVANISH = 0;
        const int MARGIN_ATTACKED = -10000;
        const int MARGIN_WITHOUTPOTENTIAL = -1000000;



        // ２回消す場合の得点計算
        // 一回目が十分大きい場合は，整理消し・ＴＦ・スマートキャンセル等にはあたらないので，１回の場合に準じる
        private int updateScoreDouble(int actionIndex, int currentBest, bool fireNow, AITool_Result result1, int turn1, AITool_Result result2, int turn2) {
            if (result1.Gain > 35) {
                return updateScoreSingle(actionIndex, currentBest, fireNow, result1, turn1);
            }
            // 相手の攻撃に綺麗に反応できる場合
            if ( (result1.Gain > fHisAttackTurn1 && fHisAttackTurn1 > 4 && turn1 == 1) ||
                 (result1.Gain > fHisAttackTurn2 && fHisAttackTurn2 > 4 && turn1 <= 2) ||
                 (result1.Gain > fHisAttackTurn3 && fHisAttackTurn3 > 4 && turn1 <= 3)    ) {
                int score = MARGIN_RESTTURN1 + 100 * result1.Gain + 1000 * result2.Gain;
                if (currentBest < score) {
                    fSummary[actionIndex] = "Smart Response to Weak Attack = " + result1.Gain+ ", potential keep = " + result2.Gain;
                    return score;
                } else {
                    return currentBest; // この条件満たしててベスト更新できないなら，以下の条件を満たしても無理
                }
            }

            // こっちからちょっかい出す場合
            // 攻撃があるなら，１回目なしのものと比較して７～８割程度のものがキープできてればよい　（ＴＦ）
            // 攻撃がなくても，１回目なしのものと比較して大きくなるならよい（整理）
            // ただし，攻撃自体が資産減少＆ワンテンポ遅れるので，ある程度のペナルティあり
            if (fHisAttackTurn3 == 0) {
                if (result1.Gain > 6) {
                    int score = MARGIN_NORMALPOTENTIAL + fRand.Next(150) + result1.Gain * 100 + result2.Gain * 125;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Tiger Fox = " + result1.Gain + ", potential keep = " + result2.Gain;
                        return score;
                    }
                } else {
                    int score = MARGIN_NORMALPOTENTIAL_CLEARNING + fRand.Next(150) + result2.Gain * 95;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Cleaning = " + result1.Gain + ", potential keep = " + result2.Gain;
                        return score;
                    }
                }
                // この条件満たしててベスト更新できなくても，別の条件で更新する可能性があるので　else return currentBest とはしない
            }

            // どちらにも当てはまらない場合は，１回の場合に準じる
            return updateScoreSingle(actionIndex, currentBest, fireNow, result1, turn1);
        }

        // １回消す場合の得点計算
        // 初手で消すのは普通悪い．例外は，十分大きい攻撃ができるときと，敵の攻撃まで１手しかないとき
        // 初手で消さない場合は，普通のポテンシャルなので，最大になるようにする
        private int updateScoreSingle(int actionIndex, int currentBest, bool fireNow, AITool_Result result1, int turn1) {
            if (fireNow == true) {
                // 初手で消すが，十分大きい場合
                if (result1.Gain > 450) {
                    int score = MARGIN_ENOUGHATTACK + 100*result1.Gain;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Sufficient Attack = " + result1.Gain;
                        return score;
                    } else {
                        return currentBest;
                    }
                }
                // 初手で消すが，敵の攻撃まで１手しかなくてやむをえないとき
                if (fHisAttackTurn1 > 4) {
                    int score = MARGIN_RESTTURN1 + 100*result1.Gain;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Response to Weak Attack = " + result1.Gain;
                        return score;
                    } else {
                        return currentBest;
                    }
                } else {
                    // 初手で消してしまうとき
                    int score = MARGIN_WITHOUTPOTENTIAL - 100*result1.Gain;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Bad Fire, the left is valueless = " + result1.Gain;
                        return score;
                    } else {
                        return currentBest;
                    }
                }
            } else {
                // 初手で消さないので普通のポテンシャル．これが最大になるようにするのが普通の戦略
                // 例外として，相手から攻撃があって間に合わない場合はペナルティ
                // １連鎖だけのときは noVanish を選ぶ
                if ((fHisAttackTurn1 > 4 && turn1 >= 2) || (fHisAttackTurn2 > 4 && turn1 >= 3) || (fHisAttackTurn3 > 4 && turn1 == 4)) {
                    int score = MARGIN_ATTACKED + fRand.Next(150) + result1.Gain * 100;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Attacked, may be too late, potential = " + result1.Gain;
                        return score;
                    } else {
                        return currentBest;
                    }
                } else if (result1.Chain == 1) {
                    return currentBest;
                } else {
                    int score = MARGIN_NORMALPOTENTIAL + fRand.Next(150) + result1.Gain * 100;
                    if (currentBest < score) {
                        fSummary[actionIndex] = "Charging, potential = " + result1.Gain;
                        return score;
                    } else {
                        return currentBest;
                    }
                }
            }
        }
    }
}

