﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace PojeClient {
    // 配珠をどこにどう落とすか．Stateに対する回答．ResignやStopGameも含む．
    class Action {
        public const int RESIGN = -1;
        public const int STOP = -2;

        private Board fBoard;  // どのボードのActionなのか
        private int fX;   //中心珠の横位置．1から始まる．真ん中あたりがデフォルト．
        private int fR;   //２個目の珠の角度．0＝右，1=上, 2=左, 3=下．3がデフォルト．
        private int fValue; // fX, fR を 一つにまとめたもの，あるいは RESIGN, STOP

        private int[] fColors;         // 配珠の色
        private Point[] fDropPoints;  // そのまま落とすと，それぞれどの座標に落ちるのか
        private bool fLegal;          // 許されるActionか

        // Resign や Stop など，値だけのコンストラクタ
        public Action(int value) {
            fValue = value;
        }

        // 位置と回転を指定したコンストラクタ
        public Action(int x, int r, Board board, int col1, int col2) {
            fBoard = board;
            fX = x;
            fR = r;
            fColors = new int[2] { col1, col2 };
            // 落ちる場所その他を計算
            calculateDropPoints();
        }

        // デフォルト位置のコンストラクタ．この位置に落とせるかどうかは気にしない．
        public Action(Board board, int col1, int col2) {
            fBoard = board;
            fX = (Settings.BoardWidth / 2)+1;
            fR = 1;  // 軸の上にある状態
            fColors = new int[2] { col1, col2 };
            // 落ちる場所その他を計算
            calculateDropPoints();
        }

        // 新しい行動を，前の行動とは別に，位置を変えて生成する．人間用．位置はみだしは修正する．落とせるかどうかは気にしない．
        public Action(Action src, int x, int r) {
            fX = x;
            fR = r;
            fColors = new int[2] { src.getColor(0), src.getColor(1) };
            fBoard = src.getBoard();

            if (fX == 0 || fX + Settings.RotateX[fR] == 0) fX++;
            if (fX == Settings.BoardWidth + 1 || fX + Settings.RotateX[fR] == Settings.BoardWidth + 1) fX--;

            calculateDropPoints();
        }


        public int getX() {
            return fX;
        }

        public int getR() {
            return fR;
        }

        public int getValue() {
            return fValue;
        }


        public Board getBoard() {
            return fBoard;
        }

        public int getColor(int i) {
            return fColors[i];
        }

        public Point[] getDropPoints() {
            return fDropPoints;
        }

        public bool isLegal() {
            return fLegal;
        }

        public String toBriefString() {
            String s = "X" + fX + " R" + fR + " pos(" + fDropPoints[0].X + "," + fDropPoints[0].Y+")(" + fDropPoints[1].X +"," +fDropPoints[1].Y+") col "+fColors[0]+" "+fColors[1];
            return s;
        }

        public static int StrToIntR(String s) {
            int r = -1;
            for (int i = 0; i < 4; i++) {
                if (s.Equals(Settings.RotateChars[i]) == true) r = i;
            }
            if (r == -1) Logger.show("Action.StrToIntR parse Error " + s);
            return r;
        }



        // 落ちる位置を計算する他
        private void calculateDropPoints() {
            fValue = 4*fX + fR;

            // １個だけの場合
            if (fColors[1]==0) {
                fDropPoints = new Point[1];
                fDropPoints[0] = new Point(fX, fBoard.getHeight(fX)+1);
                if (fDropPoints[0].Y > Settings.BoardHeight) {
                    fLegal = false;
                } else {
                    fLegal = true;
                }
                return;
            }

            // ２個ある場合
            fDropPoints = new Point[2];

            fDropPoints[0] = new Point(fX, fBoard.getHeight(fX)+1);
            int x2 = fX + Settings.RotateX[fR];
            fDropPoints[1] = new Point(x2, fBoard.getHeight(x2)+1);

            if (fR == 1) {
                // 縦で，２個目が上側
                fDropPoints[1].Y ++;
            } else if (fR == 3) {
                // 縦で，１個目が上側
                fDropPoints[0].Y++;
            }

            if (fDropPoints[0].Y > Settings.BoardHeight && fDropPoints[1].Y > Settings.BoardHeight) {
                fLegal = false;
            } else {
                fLegal = true;
            }
        }





    }
}
