﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace PojeClient {
    // Boardに関する描画をするクラス
    class BoardPainter {



        private Board fBoard;
        private PictureBox fPicBox_Board, fPicBox_Next;
        private Graphics fGraphics_Board, fGraphics_Next;
        

        // コンストラクタ．Ｉｍａｇｅも作る
        public BoardPainter(PictureBox picBox_board, PictureBox picBox_next) {
            picBox_next.Height = 30 - 20 * ((Settings.NextOpenTurns-1) % 2) + 70 * ((Settings.NextOpenTurns-2) / 2) + 2*DrawTools.stoneSize + 10;

            picBox_board.Image = new Bitmap(picBox_board.Width, picBox_board.Height);
            picBox_next.Image = new Bitmap(picBox_next.Width, picBox_next.Height);
            fPicBox_Board = picBox_board;
            fPicBox_Next = picBox_next;
            fGraphics_Next = Graphics.FromImage(fPicBox_Next.Image);
            fGraphics_Board = Graphics.FromImage(fPicBox_Board.Image);
        }

        // Boardの参照のセット
        public void setBoard(Board board) {
            fBoard = board;
        }


        // Next珠の表示
        public void showNext() {
            int[] nexts = fBoard.getNextColors();
            // 黒く塗りつぶし
            fGraphics_Next.FillRectangle(DrawTools.colorBrushes[0], 0, 0, fPicBox_Next.Width, fPicBox_Next.Height);
            // 珠表示
            for (int i = 1; i < Settings.NextOpenTurns; i++) {
                int x = 40 - 30 * (i % 2);
                int y = 30 - 20 * (i % 2) + 70 * ((i - 1) / 2);
                DrawTools.drawStone(fGraphics_Next, x, y, nexts[i * 2 + 1]);
                DrawTools.drawStone(fGraphics_Next, x, y + DrawTools.stoneSize, nexts[i * 2]);
            }
            if (Settings.TOTALWAIT>0) fPicBox_Next.Refresh();
        }

        // Actionの表示．fCurrentActionを消して新しいのを描く．
        public void showAction(Action newAction) {
            int sz = DrawTools.stoneSize;

            for (int phase = 0; phase <= 1; phase++) {
                Action act = fBoard.getShownAction();
                if (phase == 1) act = newAction;

                if (act != null) {
                    int x = act.getX();
                    int r = act.getR();

                    // 上部の消去と表示
                    int gx1 = sz * x;
                    int gy1 = sz * 3;
                    int gx2 = sz * (x + Settings.RotateX[r]);
                    int gy2 = sz * (3 - Settings.RotateY[r]);
                    if (phase == 0) {
                        DrawTools.delStone(fGraphics_Board, gx1, gy1);
                        DrawTools.delStone(fGraphics_Board, gx2, gy2);
                    } else {
                        DrawTools.drawStone(fGraphics_Board, gx1, gy1, newAction.getColor(0));
                        DrawTools.drawStone(fGraphics_Board, gx2, gy2, newAction.getColor(1));
                    }

                    // 盤上の消去と表示
                    Point[] points = act.getDropPoints();
                    for (int i = 0; i < 2; i++) {
                        if (phase == 0) {
                            DrawTools.delStoneFromBoard(fGraphics_Board, points[i].X, points[i].Y);
                        } else {
                            DrawTools.drawPredictedStone(fGraphics_Board, points[i].X, points[i].Y, newAction.getColor(i));
                        }
                    }
                }
                fBoard.setShownAction(newAction);
            }
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }


        // 珠を落とす描画
        public void drawDrop(Action a) {
            //消す
            showAction(null);

            //盤に珠を追加
            Point[] points = a.getDropPoints();
            for (int i = 0; i < 2; i++) {
                int x = points[i].X;
                int y = points[i].Y;
                if (y <= Settings.BoardHeight) {
                    DrawTools.drawStoneOnBoard(fGraphics_Board, x, y, a.getColor(i));
                }
            }
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }

        // 珠消去，連鎖数表示
        public void drawVanish_showChain(int chain) {
            foreach (Point p in fBoard.getVanishPoints()) {
                DrawTools.delStoneFromBoard(fGraphics_Board, p.X, p.Y);
                if (fBoard.getStone(p.X, p.Y) == Board.JAMSTONE) {
                    DrawTools.drawVanishingJam(fGraphics_Board, p.X, p.Y);
                } else {
                    DrawTools.drawChainNumber(fGraphics_Board, p.X, p.Y, fBoard.getStone(p.X, p.Y), chain);
                }
            }
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }

        // 連鎖数表示を消去する
        public void drawVanish_delete() {
            foreach (Point p in fBoard.getVanishPoints()) DrawTools.delStoneFromBoard(fGraphics_Board, p.X, p.Y);
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }

        // 邪魔珠降り場所の▽
        public void showJamDropPoint() {
            int sz = DrawTools.stoneSize;
            int cx = sz * fBoard.getJamInfo().getFallIndex() + (sz / 2);
            int cy = (3 * sz) / 2;
            fGraphics_Board.FillPolygon(Brushes.Gray, new Point[3] { new Point(cx - 7, cy - 5), new Point(cx + 7, cy - 5), new Point(cx, cy + 10) });
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }

        // 邪魔珠数の表示 
        public void showJamsStock() {
            fGraphics_Board.FillRectangle(DrawTools.colorBrushes[0], 0, 0, fPicBox_Board.Width, DrawTools.stoneSize);
            fGraphics_Board.DrawString(fBoard.getJamInfo().toBriefString(), DrawTools.chainFonts[3], Brushes.Silver, 10, 5);
            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }



        // Boardの表示，Action部分は消去のみ
        public void showBoard() {
            int sz = DrawTools.stoneSize;
            int wd = Settings.BoardWidth;
            int hg = Settings.BoardHeight;


            // 黒く塗りつぶし
            fGraphics_Board.FillRectangle(DrawTools.colorBrushes[0], 0, 0, fPicBox_Board.Width, fPicBox_Board.Height);
            // 枠線
            Point[] points = new Point[4] { new Point(sz, sz * 2), new Point(sz, sz * (5 + hg)), new Point(sz * (1 + wd), sz * (5 + hg)), new Point(sz * (1 + wd), sz * 2) };
            fGraphics_Board.DrawLines(Pens.White, points);
            // 縦線
            for (int x = 2; x <= wd; x++) {
                fGraphics_Board.DrawLine(Pens.DarkGray, sz * x, sz * 5, sz * x, sz * (5 + hg));
            }
            // 盤上の珠
            for (int x = 1; x <= wd; x++) for (int y = 1; y <= hg; y++) {
                DrawTools.drawStoneOnBoard(fGraphics_Board, x, y, fBoard.getStone(x, y));
            }
            // 邪魔珠ストック

            showJamDropPoint();

            if (Settings.TOTALWAIT > 0) fPicBox_Board.Refresh();
        }

        public void refresh() {
            fPicBox_Board.Refresh();
            fPicBox_Next.Refresh();
        }

    }
}
