﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace PojeClient {
    class Player_User : Player {
        private int fMyIndex;
        private int fTimeLimit;
        private Action[] fAnswerBox;
        private Form_main fOwnerForm;
        private BoardPainter fPainter;


        public const int KEY_RESIGN = -1;
        public const int KEY_STOP = -2;

        public const int KEY_NONE = 0;
        public const int KEY_LEFT = 1;
        public const int KEY_RIGHT = 2;
        public const int KEY_ROTRIGHT = 3;
        public const int KEY_ROTLEFT = 4;
        public const int KEY_DROP = 5;

        public const int KEY_REPLAYSTOP = 10;
        public const int KEY_REPLAYNEXT = 11;
        public const int KEY_REPLAYAUTO = 12;
        public const int KEY_REPLAYTOUCH = 13;
        public const int KEY_REPLAYTO = 14;
        public const int KEY_REPLAYBACK = 15;





        public Player_User(Form_main owner) {
            fOwnerForm = owner;
        }

        public void setPainter(BoardPainter painter) {
            fPainter = painter;
        }

        public String getName() {
            return "User_"+Settings.userName;
        }

        public void newGame(int yourIndex) {
            fMyIndex = yourIndex;
        }

        public void endGame(int result) {
        }


        public void setParam( int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
            fOwnerForm.clearPushedKey();
        }

        public void makeAction(object currentState) {
            State state = (State)currentState;
            Board myBoard = state.getBoard(fMyIndex);

            while (true) {
                // 一定間隔でキーの状態を得る
                Thread.Sleep(20);
                Application.DoEvents();

                if (Signal.FormClosed == true) return;

                int key = fOwnerForm.getPushedKey();

                if (key == KEY_NONE) continue;

                if (key == KEY_DROP) {
                    fOwnerForm.clearPushedKey();  // 多分不要だが念のため
                    break;       // 終了＝着手する
                }

                Action newact = null;
                if (key == KEY_LEFT) {
                    newact = new Action(fAnswerBox[0], fAnswerBox[0].getX() - 1, fAnswerBox[0].getR());
                } else if (key == KEY_RIGHT) {
                    newact = new Action(fAnswerBox[0], fAnswerBox[0].getX() + 1, fAnswerBox[0].getR());
                } else if (key == KEY_ROTRIGHT) {
                    newact = new Action(fAnswerBox[0], fAnswerBox[0].getX(), (fAnswerBox[0].getR() + 3) % 4);   // 右回転
                } else if (key == KEY_ROTLEFT) {
                    newact = new Action(fAnswerBox[0], fAnswerBox[0].getX(), (fAnswerBox[0].getR() + 1) % 4);   // 左回転
                }


                if (key == KEY_STOP) {
                    newact = new Action(Action.STOP);
                    fAnswerBox[0] = newact;      // 置き換え
                    fOwnerForm.clearPushedKey();
                    break;
                }
                if (key == KEY_RESIGN) {
                    newact = new Action(Action.RESIGN);
                    fAnswerBox[0] = newact;      // 置き換え
                    fOwnerForm.clearPushedKey();
                    break;
                }

                if (newact != null) {
                    fPainter.showAction(newact);  // 描画
                    fAnswerBox[0] = newact;      // 置き換え
                    fOwnerForm.clearPushedKey();
                    continue;
                }
            }
        }

    }
}
